package com.lovense.sdkdemo

import android.Manifest
import android.annotation.SuppressLint
import android.bluetooth.BluetoothAdapter
import android.content.Context
import android.content.Intent
import android.content.pm.PackageManager
import android.location.LocationManager
import android.os.Build
import android.os.Bundle
import android.util.Log
import android.widget.Toast
import androidx.appcompat.app.AppCompatActivity
import androidx.core.app.ActivityCompat
import androidx.recyclerview.widget.LinearLayoutManager
import com.lovense.sdkdemo.databinding.ActivityMainBinding
import com.lovense.sdklibrary.Lovense
import com.lovense.sdklibrary.LovenseToy
import com.lovense.sdklibrary.callBack.LovenseError
import com.lovense.sdklibrary.callBack.OnSearchToyListener
import com.tbruyelle.rxpermissions3.RxPermissions
import org.greenrobot.eventbus.EventBus
import org.greenrobot.eventbus.Subscribe
import org.greenrobot.eventbus.ThreadMode

open class MainActivity : AppCompatActivity() {
    private var rxPermissions: RxPermissions? = null
    private var toyList: MutableList<LovenseToy> = ArrayList()
    private var toyAdapter: ToyAdapter? = null

    private val TAG = MainActivity::class.java.simpleName
    private lateinit var binding: ActivityMainBinding
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        binding = ActivityMainBinding.inflate(layoutInflater)
        setContentView(binding.root)

        EventBus.getDefault().register(this)
        Lovense.getInstance(application).setDeveloperToken("Your Token")
        binding.btnToken.setOnClickListener {
            Lovense.getInstance(application).setDeveloperToken(binding.etToken.text.toString())
        }

        binding.startScan.setOnClickListener { requestPermissions() }
        binding.stopScan.setOnClickListener {
            Lovense.getInstance(application).stopSearching()
            binding.title.text = "search toy(stop scan)"
        }
        binding.title.setOnClickListener {
            Lovense.getInstance(application).listToys { lovenseError ->

            }
        }

        val manager = LinearLayoutManager(this, LinearLayoutManager.VERTICAL, false)
        toyAdapter = ToyAdapter(this, toyList)
        binding.recylerView.layoutManager = manager
        binding.recylerView.adapter = toyAdapter

        rxPermissions = RxPermissions(this)
    }

    private fun scanDev() {
        toyList.clear()
        toyAdapter?.notifyDataSetChanged()
        Toast.makeText(this@MainActivity, "start scan！", Toast.LENGTH_SHORT).show()
        binding.title.text = "search toy(scaning)"

        Lovense.getInstance(application).searchToys(object : OnSearchToyListener {
            override fun onSearchToy(toy: LovenseToy) {
                runOnUiThread { addDevice(toy) }
            }

            override fun finishSearch() {
                Lovense.getInstance(application).saveToys(toyList) { }
                binding.title.text = "search toy(stop scan)"
            }

            override fun onError(msg: LovenseError) {
                Toast.makeText(this@MainActivity, msg.message, Toast.LENGTH_SHORT).show()
            }
        })
    }

    private fun addDevice(toy: LovenseToy?) {
        if (toy != null && !isAdded(toy.toyId)) {
            toyList.add(toy)
            toyAdapter?.notifyDataSetChanged()
        }
    }

    private fun isAdded(toyId: String?): Boolean {
        return toyList.any { !it.toyId.isNullOrEmpty() && it.toyId == toyId }
    }

    @Subscribe(threadMode = ThreadMode.MAIN)
    fun onMessageEvent(mToyConnectEvent: ToyConnectEvent) {
        // 只有当适配器不为null时，才进行处理
        toyAdapter?.let { adapter ->
            val index = toyList.indexOfFirst { it.toyId == mToyConnectEvent.id }
            if (index != -1) {
                toyList[index].status = mToyConnectEvent.connect
                adapter.notifyItemChanged(index)
            }
        }
    }

    override fun onDestroy() {
        super.onDestroy()
        EventBus.getDefault().unregister(this)
    }

    private fun requestPermissions() {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.S) {
            handleBluetoothPermissions(
                arrayOf(Manifest.permission.BLUETOOTH_SCAN, Manifest.permission.BLUETOOTH_CONNECT),
                "Android 12.0 and above requires access to bluetooth permissions to connect Bluetooth devices. In order to connect your toys, please grant the app bluetooth permissions."
            )
        } else if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            handleBluetoothPermissions(
                arrayOf(Manifest.permission.ACCESS_FINE_LOCATION, Manifest.permission.BLUETOOTH),
                "If you're using Android 6.0+, your GPS must be enabled to connect to Bluetooth devices."
            )
        } else {
            scanDev()
        }
    }

    @SuppressLint("CheckResult")
    private fun handleBluetoothPermissions(permissions: Array<String>, errorMessage: String) {
        val noPermission = permissions.any { ActivityCompat.checkSelfPermission(this@MainActivity, it) != PackageManager.PERMISSION_GRANTED }
        // 没有权限，申请权限，返回
        if (noPermission) {
            rxPermissions?.request(*permissions)?.subscribe { granted ->
                if (granted) {
                    requestBTAction()
                } else {
                    Toast.makeText(this@MainActivity, errorMessage, Toast.LENGTH_SHORT).show()
                }
            }
        } else {
            scanDev()
        }
    }

    private fun requestBTAction() {
        if (!checkBTEnable(this, true)) {
            Toast.makeText(this@MainActivity, "Please open the bluetooth", Toast.LENGTH_SHORT).show()
            return
        }
        if (Build.VERSION.SDK_INT in Build.VERSION_CODES.M..Build.VERSION_CODES.R && !isOPenGps(this)) {
            Toast.makeText(this@MainActivity, "Please open the gps.", Toast.LENGTH_SHORT).show()
            return
        }
        scanDev()
    }

    /**
     * isEnable Bluetooth
     */
    private fun checkBTEnable(context: Context, promptEnable: Boolean): Boolean {
        val bluetoothAdapter = BluetoothAdapter.getDefaultAdapter()
        if (bluetoothAdapter == null) {
            Log.d(TAG, "not support bluetooth")
            return false
        }
        if (bluetoothAdapter.isEnabled) {
            return true
        }
        if (promptEnable && ActivityCompat.checkSelfPermission(this, Manifest.permission.BLUETOOTH_CONNECT) == PackageManager.PERMISSION_GRANTED) {
            val enableIntent = Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE)
            enableIntent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
            context.startActivity(enableIntent)
        }
        return false
    }

    /**
     * isStarted GPS or AGPS
     *
     * @param context
     * @return true isStarted
     */
    private fun isOPenGps(context: Context): Boolean {
        val locationManager = context.getSystemService(LOCATION_SERVICE) as LocationManager
        // 通过GPS卫星定位，定位级别可以精确到街（通过24颗卫星定位，在室外和空旷的地方定位准确、速度快）
        val gps = locationManager.isProviderEnabled(LocationManager.GPS_PROVIDER)
        // 通过WLAN或移动网络(3G/2G)确定的位置（也称作AGPS，辅助GPS定位。主要用于在室内或遮盖物（建筑群或茂密的深林等）密集的地方定位）
        val network = locationManager.isProviderEnabled(LocationManager.NETWORK_PROVIDER)
        return (gps || network)
    }
}
